use strict;
use threads;
use threads::shared;
use Test::Simple tests => 23;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Log qw(InitLog);
use DE_EPAGES::Core::API::Object::MonitorShared;
use DE_EPAGES::Core::API::Object::MonitorThread;
use DE_EPAGES::Core::API::Object::ProgressMonitor;

local $| = 1;
InitLog();

my $time1 = time;
my $SharedMonitor : shared = DE_EPAGES::Core::API::Object::MonitorShared->new;
ok (1, sprintf('SharedMonitor started: %d sec.', time - $time1));
ok (!$SharedMonitor->isStarted, 'new monitor is not started');
ok (!$SharedMonitor->isBlocked, 'new monitor is not blocked');

my $MonitorThread = DE_EPAGES::Core::API::Object::MonitorThread->new(
    'Monitor' => $SharedMonitor,
    'Timeout' => 2,
);
ok (1, sprintf('MonitorThread started: %d sec.', time - $time1));

my $TestVar = 0;
$MonitorThread->run( Sub => sub { $TestVar = 1 } );
ok( 1 == $TestVar, 'very fast sub' );
ok( !$SharedMonitor->isStarted, 'monitor not started' );

$MonitorThread->run( Sub => sub { sleep 1; $TestVar = 2 } );
ok( 2 == $TestVar, 'slow sub' );
ok( !$SharedMonitor->isStarted, 'monitor not started' );

$MonitorThread->run( Sub => sub { $MonitorThread->startMonitor; $TestVar = 3 } );
ok( 3 == $TestVar, 'manual monitor' );
ok( $SharedMonitor->isStarted, 'monitor started manually' );

$MonitorThread->run( Sub => sub { $SharedMonitor->blockTimeOut; sleep 3; $TestVar = 4 } );
ok( 4 == $TestVar, 'very slow sub, blocked monitor' );
ok( !$SharedMonitor->isStarted, 'monitor blocked' );
ok( $SharedMonitor->isBlocked, 'monitor blocked flag set' );

$MonitorThread->run( Sub => sub { sleep 4; $TestVar = 5 } );
ok( 5 == $TestVar, 'very slow sub' );
ok( $SharedMonitor->isStarted, 'monitor started' );
ok( !$SharedMonitor->isBlocked, 'monitor blocked was reset' );

my $ProgressMonitor = DE_EPAGES::Core::API::Object::ProgressMonitor->new(
   'ContentTemplate' => "# %4d of %4d products exported.\n",
   'Total'          => 20,
   'Current'        => 0,
   'SharedMonitor'  => $SharedMonitor,
   'MonitorThread'  => $MonitorThread,
);
ok (1, sprintf('ProgressMonitor started: %d sec.', time - $time1));

$MonitorThread->run(
    Sub => sub {
        $MonitorThread->startMonitor();
        for(my $i=0;$i<5;$i++) {
            ok( 1, "Item $i" );
            $ProgressMonitor->incCurrent;
#            $ProgressMonitor->write;
            sleep(1);
        }
    }
);

# calls DESTROY and stops the thread
undef $MonitorThread;
ok( 1, 'delete MonitorThread' );
