use strict;
use Time::HiRes qw (time sleep);
use threads;
use threads::shared;
use Thread::Semaphore;

my $IsRunning = Thread::Semaphore->new(0);
my $IsMonitorEnd = Thread::Semaphore->new(0);
my $WaitForEnding : shared = 1;
my $IsEnded : shared;

my @LongSubTimes = (7, 0.3, 0.4, 0.5, 0.6, 0.7, 0.1, 0.2, 0.3, 10, 3, 7); ##no critic qw(ProhibitMagicNumbers)
my $TimeToSleep  = 4;
my @TimeOuts     = (1,   0,   0,   0,   0,   0,   0,   0,   0, 1, 0, 1);

local $| = 1;

print "1..14\n";

# simple test framework for multi-thread tests
my $OK_COUNTER : shared = 0;
sub ok {
    my( $Check, $Message ) = @_;

    if( $Check ) {
        $OK_COUNTER++;
        print "ok $OK_COUNTER - $Message\n";
    }
    else {
        print "not ok $OK_COUNTER - $Message\n";
    }
}

sub Monitor {
    my $Counter = 0;
    # waits request starts IsRunning == 1 then reduce to 0,
    # the other thread increments this semaphore if a new request starts
    $IsRunning->down;
    while ($WaitForEnding) {

        # set lock to IsEnded, waits until request ends or time to sleep is over
        my $time= time;
        my $res;
        {
            lock $IsEnded;
            $res = cond_timedwait($IsEnded, $time + $TimeToSleep);
        }
        ok ((     defined $res and not $TimeOuts[$Counter] or
              not defined $res and     $TimeOuts[$Counter]), "step $Counter sleeps " . (time - $time));

        # increment counter for test display
        $Counter++;
        $IsMonitorEnd->up;
        $IsRunning->down;
    }
    ok (1, 'monitor closed');
    return;
}

sub veryLongSub {
    my ($Duration) = @_;
    sleep($Duration);
}

my $monitor = threads->new(\&Monitor);
foreach (@LongSubTimes) { # simulate requests
    # send request starts to monitor IsRunning = 1
    $IsRunning->up;

    # this is a request, which can be longer then expected.
    veryLongSub($_);

    # send end of request signal to monitor
    {
        lock $IsEnded;
        cond_signal $IsEnded;
    }
    # wait till monitor ends
    $IsMonitorEnd->down;
}

# send close signal to monitor (IsRunning > 1)
$WaitForEnding = 0;
$IsRunning->up;
$monitor->join;
ok( 1, 'threads all closed' );
