#!/bin/bash
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBN8drul5jMabHSgyARAmPrAJ9t+o5QomysHvGzON53yWCi+JLGBgCgk1cv
#o8T8Z1xSQcMG2UfU9E22VbE=
#=wTtF
#-----END PGP SIGNATURE-----
#line 2

EPOCH=`date "+%s"`

Usage () {
  [ -n "$USAGE" ] && return
  echo "Usage:"
  echo "  . configure_epages.sh [-p RPM_INSTALL_PREFIX]"
  echo "Options:"
  echo "  -p: EPAGES parent dir (default: dirname \$EPAGES)"
  echo "     (EPAGES parent dir must contain etc/ and eproot/)"
}

[ ! -f /etc/sysconfig/epages6 ] || . /etc/sysconfig/epages6
EPAGES_PROJECT=${EPAGES_PROJECT:-$EPAGES_INSTALL_RUN_SCRIPT}

unset RPM_INSTALL_PREFIX
[ "x$1" = "x-p" ] && RPM_INSTALL_PREFIX=$2
[ -z "$RPM_INSTALL_PREFIX" -a -f /etc/default/"$EPAGES_PROJECT" ] && {
  . /etc/default/"$EPAGES_PROJECT"
  [ -n "$EPAGES" ] && RPM_INSTALL_PREFIX=`dirname "$EPAGES"`
}

RPM_INSTALL_PREFIX=`cd "$RPM_INSTALL_PREFIX" 2>/dev/null && pwd`

if [ -z "$RPM_INSTALL_PREFIX" -o ! -d "$RPM_INSTALL_PREFIX"/eproot ] ; then
  Usage
else

###############################################################################
# INTERNAL FUNCTIONS
###############################################################################

backup_file () {
  [ -n "$VERBOSE" ] && echo "backup_file $@" 1>&2
  local FILE=${1%.ep[tx]*}
  if [ -f "$FILE" ] ; then
    # backup file
    cp -p "$FILE" "$FILE".~$EPOCH~
  elif [ -n "$2" ] ; then
    # create empty file with same rights as $2
    cp -p "$2" "$FILE"
    cat /dev/null > "$FILE"
  elif [ "$1" != "$FILE" ] ; then
    # copy FILE.ept to FILE
    cp -p "$1" "$FILE"
  fi
}

normalize_IPv6_address () {
  [ -z "$1" ] && return
  local IP=$1
  local PRE=$(echo ${IP%::*})
  local POST=$(echo ${IP#*::})
  local PART_MAX=8
  [ "${IP/./}" = "$IP" ] || PART_MAX=7

  if [ "$PRE" != "$IP" ] ; then
    local PRE_PARTS=0
    [ -n "$PRE" ] && { PRE_PARTS=$(echo "$PRE" | tr ':' '\n' | wc -l); PRE="$PRE:"; }
    local POST_PARTS=0
    [ -n "$POST" ] && {  POST_PARTS=$(echo "$POST" | tr ':' '\n' | wc -l); POST=":$POST"; }
    local IN_PARTS=$(($PART_MAX - $PRE_PARTS - $POST_PARTS))
    local NULL=0
    for i in $(seq 2 $IN_PARTS) ; do NULL="$NULL:0" ; done
    IP="$PRE$NULL$POST"
  fi

  echo "$IP" | sed 's,:0\+\([^:]\),:\1,g;s,^0\+\([^:]\),\1,' | tr A-Z a-z
}

get_netaddr () {
  [ -z "$1" ] && return
  local IP=$1
  local USE_IPv6=$2
  local PART_MAX=5

  if [ -z "${IP//[^:]/}" ] ; then
    # quote IPv4 address
    IP=${IP//\./\\\\.}
    echo -n "${IP%.*}."'\\*\ '
  else
    # normalize IPv6 address
    IP=$(normalize_IPv6_address $IP)
    if [ "$IP" != "0:0:0:0:0:0:0:1" ] ; then
      [ "${IP/./}" = "$IP" ] && PART_MAX=5 || PART_MAX=7
      set -- `echo ${IP//:/ }`
      IP=${@:0:$PART_MAX}
      echo -n ${IP// /:}:'\\*\ '
    fi
  fi

  for i in $(LANG=C ifconfig | awk '/(Bcast:|Scope:Global)/{sub(/.* addr: */,"");sub(/ .*/,"");print}') ; do
    if [ -z "${i//[^:]/}" ] ; then
      i=${i//\./\\\\.}
      echo -n "${i%.*}."'\\*\ '
    else
      [ -z "$USE_IPv6" ] && continue
      i=$(normalize_IPv6_address $i)
      [ "${i/./}" = "$i" ] && PART_MAX=5 || PART_MAX=7
      set -- `echo ${i//:/ }`
      i=${@:0:$PART_MAX}
      echo -n ${i// /:}:'\\*\ '
    fi
  done
}

check_IP () {
  local ADDR=${1:-unknown}
  local IS_IP=no
  local IPv4=$ADDR
  local ERROR FQDN IPv6

  echo $ADDR | tr -d '[.:0-9a-z\300-\377-]' | grep -q . && { echo $ADDR $IS_IP FRMT_VOID_CHARS; return; }

  if [ -z "${ADDR//[.:0-9a-f]/}" -a "$ADDR" != "${ADDR/:/}" ] ; then
    IS_IP=v6
    set -- `echo ${ADDR//:/ }`
    IPv4=${@:$#} # last part

    if [ "${IPv4/./}" != "$IPv4" ] ; then
      [ "$IS_IP" = v4 ] && unset IPv6 || IPv6=${ADDR%$IPv4}
    else
      unset IPv4; IPv6=$ADDR
    fi
  elif [ -z "${ADDR//[.0-9]/}" ] ; then
    IS_IP=v4
  fi

  if [ -n "$IPv4" ] ; then
    # contains '..'
    [ "${IPv4/../}" != "$IPv4" ] && { echo $ADDR $IS_IP FRMT_DOTDOT; return; }
    # starts/ends with '.'
    [ "${IPv4#.}" != "$IPv4" -o "${IPv4%.}" != "$IPv4" ] && { echo $ADDR $IS_IP FRMT_BOUNDARY_DOT; return; }
  fi

  # ADDR is hostname

  if [ "$IS_IP" = no ] ; then
    # contains ':'
    [ "${ADDR/:/}" != "$ADDR" ] && { echo $ADDR $IS_IP FRMT_VOID_COLON; return; }
    # is longer than 253 chars
    [ ${#ADDR} -gt 253 ] && { echo $ADDR $IS_IP FRMT_NAME_GT_253; return; }

    type -t idn >/dev/null && HAS_IDN=1 || unset HAS_IDN
    for i in ${ADDR//./ } ; do
      # starts/ends with [-]
      [ "$i" != "${i#-}" -o "$i" != "${i%-}" ] && ERROR=FRMT_BOUNDARY_HYPHEN
      # encode punycode if hostname is IDN (it contains [\300-\377])
      if [ -n "$HAS_IDN" -a -n "${i//[0-9a-z-]/}" ] ; then
        i="xn--$(LANG=en_US idn --quiet -e "$i")"
      fi
      # is longer than 63 chars
      [ ${#i} -gt 63 ] && ERROR=FRMT_PART_GT_63
      # assemble parts
      FQDN="${FQDN:+$FQDN.}$i"
    done
    echo $FQDN $IS_IP $ERROR
    return
  fi

  # ADDR is IPv4

  if [ -n "$IPv4" ] ; then
    # doesn't contain exactly 3 '.'
    local NO_DOT=${IPv4//./}
    if [ $((${#IPv4} - ${#NO_DOT})) -ne 3 ] ; then
      [ "$IS_IP" = v6 ] && ERROR=FRMT_DOT_NE_3 || IS_IP=no
      echo $ADDR $IS_IP $ERROR
      return
    fi
    # contains characters not in [0-9.]
    if [ -n "${IPv4//[0-9.]/}" ] ; then
      [ "$IS_IP" = v6 ] && ERROR=FRMT_VOID_4CHARS || IS_IP=no
      echo $ADDR $IS_IP $ERROR
      return
    fi
    # part is > 255
    for i in ${IPv4//./ } ; do
      if [[ ${#i} -gt 3 || $i -gt 255 ]] ; then
        [ "$IS_IP" = v6 ] && ERROR=FRMT_4NUMBER_GT_255 || IS_IP=no
        echo $ADDR $IS_IP $ERROR
        return
      fi
    done
    # IPv6 part must be :: or ::ffff:
    IPv6=$(normalize_IPv6_address "$IPv6")
    [ -z "$IPv6" -o "$IPv6" = 0:0:0:0:0:0: -o "$IPv6" = 0:0:0:0:0:ffff: ] || {
      ERROR=FRMT_IPv6_PART_NOT_FFFF_OR_0
    }

    echo $ADDR $IS_IP $ERROR
    return
  fi

  # ADDR is IPv6

  # contains less than 2 ':'
  local NO_COLON=${IPv6//:/}
  [ $((${#IPv6} - ${#NO_COLON})) -lt 2 ] && { echo $ADDR $IS_IP FRMT_COLON_LT_2; return; }
  # contains ':::'
  [ "${IPv6/:::/}" != "$IPv6" ] && { echo $ADDR $IS_IP FRMT_COLONCOLONCOLON; return; }
  # contains ::.*::
  [ "${IPv6/::/}" != "${IPv6//::/}" ] && { echo $ADDR $IS_IP FRMT_COLONCOLON_COLONCOLON; return; }
  # contains more than 8 ':'
  [ $((${#IPv6} - ${#NO_COLON})) -gt 8 ] && { echo $ADDR $IS_IP FRMT_COLON_GT_8; return; }
  # doesn't contains exactly 7 ':'
  [ "${IPv6/::/}" = "$IPv6" -a $((${#IPv6} - ${#NO_COLON})) -ne 7 ] && { echo $ADDR $IS_IP FRMT_COLON_NE_7; return; }
  # starts with ':'
  [ "${IPv6#:}" != "$IPv6" -a "${IPv6#::}" = "$IPv6" ] && { echo $ADDR $IS_IP FRMT_BOUNDARY_COLON; return; }
  # ends with ':'
  [ "${IPv6%:}" != "$IPv6" -a "${IPv6%::}" = "$IPv6" ] && { echo $ADDR $IS_IP FRMT_BOUNDARY_COLON; return; }

  for i in ${IPv6//:/ } ; do
    [[ ${#i} -gt 4 ]] && { echo $ADDR $IS_IP FRMT_6NUMBER_GT_FFFF; return; }
  done

  echo $ADDR $IS_IP
}

dns_lookup () {
  [ -z "$1" ] && return
  local SERVER=$1
  local HOST="`eval echo '$'EPAGES_INSTALL_${SERVER}_HOST`"
  local IP=${HOST%:ip6}
  local ERROR FQDN ADDR IPv4 IPv6

  # use hostname if no argument
  if [ -z "$IP" ] ; then
    IP=`hostname --fqdn | head -1`
    LANG=C host "$IP" 2>/dev/null | grep -q 'has.* address' || {
      IP=`hostname | head -1`
    }
  fi
  # use localhost if hostname does not work
  IP=${IP:-localhost}
  # remove spaces from hostname/IP, lowercase
  IP=$(echo "${IP//[[:space:]]/}" | tr A-Z a-z)

  set -- `check_IP "$IP"`
  FQDN=$1; IS_IP=$2; ERROR=$3

  if [ -n "$ERROR" ] ; then
    IPv4=$(LANG=C ifconfig lo 2>/dev/null | tr -d '\n' | grep -qi 'inet addr:.*UP.*RUNNING')
    IPv6=$(LANG=C ifconfig lo 2>/dev/null | tr -d '\n' | grep -qi 'inet6 addr:.*UP.*RUNNING')

    [ \( -n "$IPv4" -o -z "$IPv6" \) -a "$HOST" = "${HOST//:/}" ] && {
      IP=127.0.0.1; FQDN=localhost; } || { IP=::1; FQDN=ipv6-localhost; }
    ADDR=$IP
  elif [ "$IS_IP" != no ] ; then
    # get hostname from IP
    set -- `LANG=C host "$IP" 2>/dev/null | egrep '(domain name pointer|^1(\.0){31}\.ip6\.arpa )'`
    line=$*
    [ -n "$1" -a "${line/domain name pointer/}" = "$line" ] && FQDN=ipv6-localhost || FQDN=${5:-$FQDN}
    [ -z "$1" ] && ERROR="NXDOMAIN_IP$IS_IP"
    [ -n "${HOST%:ip6}" ] || ADDR=${FQDN%.}
  elif [ "$HOST" = "${HOST%:ip6}" ] ; then
    # get IP from hostname (IPv4)
    set -- `LANG=C host -t A "$FQDN" 2>/dev/null | grep 'has address'`
    IP=${4:-127.0.0.1}
    [ -z "$1" ] && { ADDR=127.0.0.1; FQDN=localhost; ERROR=SERVFAIL_IPv4; } || { [ "$FQDN" = "${1%%.*}" ] && FQDN=$1; }
  else
    # get IP from hostname (IPv6)
    set -- `LANG=C host -t AAAA "$FQDN" 2>/dev/null | grep 'has IPv6 address'`
    IP=${5:-::1}
    [ -z "$1" ] && ERROR=SERVFAIL_IPv6 || { [ "$FQDN" = "${1%%.*}" ] && FQDN=$1; }
    set -- `LANG=C host -t A "$FQDN" 2>/dev/null | grep 'has address'`
    IPv4=$4
    if [ -n "$ERROR" ] ; then
      IP=${IPv4:-::1}
      unset IPv4
      [ -z "$1" ] && { ADDR=::1; FQDN=ipv6-localhost; ERROR=SERVFAIL_IPv4v6; } || { [ "$FQDN" = "${1%%.*}" ] && FQDN=$1; }
    fi
  fi
  ADDR=${ADDR:-${HOST%:ip6}}
  ADDR=${ADDR:-$FQDN}

  eval EPAGES_INSTALL_${SERVER}_ADDR=$(echo "$ADDR" | tr A-Z a-z)
  eval EPAGES_INSTALL_${SERVER}_NET=$(get_netaddr "$IP" "${HOST//[^:]/}")
  eval EPAGES_INSTALL_${SERVER}_IP=$IP
  eval EPAGES_INSTALL_${SERVER}_IPv4=$IPv4
  eval EPAGES_INSTALL_${SERVER}_FQDN=${FQDN%.}
  eval EPAGES_INSTALL_${SERVER}_ERROR=$ERROR
}

is_positive () {
  [ -n "$VERBOSE" ] && echo "is_positive $@" 1>&2
  # bash can't compute numbers >= 1024*1024*1024*1024*1024*1024*8
  [ -z "$1" -o -n "${1//[0-9]/}" -o ${#1} -gt 18 ] && return 1
  return $(($1 < 1))
}

repair_16bit () {
  local MIN=1
  [ "x$1" = "x-u" ] && { MIN=1024; shift; } 
  local ID=${1//[^0-9]/}
  # bash can't compute numbers >= 1024*1024*1024*1024*1024*1024*8
  [ -z "$ID" -o ${#ID} -gt 18 ] && { echo $2; return; }
  [ $ID -lt $MIN -o $ID -gt 65535 ] && echo $2 || echo $ID
}

repair_account () {
  local NODOLLAR
  [ "x$1" = "x-d" ] && { NODOLLAR=1; shift; } 
  # remove all characters not in [A-Za-z0-9_.$-]
  local ID=`echo "$1" | tr -d -c '[A-Za-z0-9_.\$-]'`
  [ -n "$NODOLLAR" ] && ID=${ID//\$/}
  # remove leading [.$-]
  while [ "$ID" != "${ID/#[.\$-]/}" ] ; do ID=${ID/#[.\$-]/} ; done
  [[ "$ID" == *'$' ]] && DOLLAR='$' || unset DOLLAR
  # remove all [$] but not trailing
  ID=${ID//[\$]/}"$DOLLAR"
  # account name must not have more than 32 characters
  [ -n "$ID" ] && echo "${ID:0:32}" || echo $2
}

local_timezone () {
  # 1st try: environment variable TZ
  [ -n "$TZ" ] && { echo "$TZ" ; return; }

  # 2nd try: compare files in /usr/share/zoneinfo with /etc/localtime
  if [ -f /etc/localtime ] && cd /usr/share/zoneinfo 2>/dev/null ; then
    find [A-Z]*/* -maxdepth 0 -type f 2>/dev/null | sed 's,^Etc/, Etc/,' | sort -r \
    | while read i ; do
      while [ "$i" != "${i# }" ] ; do i=${i# } ; done
      if cmp -s "$i" /etc/localtime ; then echo "$i"; exit 1; fi
    done || return
  fi

  # 3rd try: /etc/sysconfig/clock
  if [ -f /etc/sysconfig/clock ] ; then
    unset TIMEZONE ZONE
    . /etc/sysconfig/clock
    if grep -q TIMEZONE /etc/init.d/boot.clock 2>/dev/null ; then
      [ -n "$TIMEZONE" ] && { echo "$TIMEZONE"; return; }
    else
      [ -n "$ZONE" ] && { echo "$ZONE"; return; }
      [ -n "$TIMEZONE" ] && { echo "$TIMEZONE"; return; }
    fi
  fi
}

###############################################################################
# VARIABLES                                                                   #
###############################################################################

unset EPAGES_INSTALL_DEFINED_HOST
[ -n "$EPAGES_INSTALL_AS_HOST" ] && EPAGES_INSTALL_DEFINED_HOST=1
[ -n "$EPAGES_INSTALL_RR_HOST" ] && EPAGES_INSTALL_DEFINED_HOST=1

EPAGES="$RPM_INSTALL_PREFIX"/eproot
INI_CONFIG="$EPAGES"/bin/ini-config.sh
FILE="$RPM_INSTALL_PREFIX"/etc/default/epages6

if [ -f $FILE ] ; then
  for i in EPAGES_PROJECT EPAGES_APPUSER EPAGES_WEBUSER EPAGES_WEBGROUP ; do
    CURRENT=$("$INI_CONFIG" -f "$FILE" -g -k $i 2>/dev/null)
    if [[ -n $CURRENT ]] ; then
     case $i in
       EPAGES_PROJECT) EPAGES_INSTALL_RUN_SCRIPT=$CURRENT ;;
       EPAGES_APPUSER) EPAGES_INSTALL_AS_RUNUSER=$CURRENT ;;
       EPAGES_WEBUSER) EPAGES_INSTALL_WS_RUNUSER=$CURRENT ;;
       EPAGES_WEBGROUP) EPAGES_INSTALL_WS_GROUP=${CURRENT%:*} ;;
     esac
    fi
  done
fi

#+++++++++++++++++++++++++++++#
#      USER/GROUP OPTIONS     #
#+++++++++++++++++++++++++++++#

EPAGES_INSTALL_WS_GROUP=`repair_account "$EPAGES_INSTALL_WS_GROUP"`

if [ -n "$EPAGES_INSTALL_WS_GROUP" ] ; then
  EPAGES_INSTALL_WS_GID=`awk -F: '{if ($1 == ENVIRON["EPAGES_INSTALL_WS_GROUP"]) {print $3; exit 0}}' /etc/group`
else
  EPAGES_INSTALL_WS_GID=`awk -F: '{if ($1 == "ep_web") {print $4; exit 0}}' /etc/passwd`
  EPAGES_INSTALL_WS_GROUP=`awk -F: '{if ($3 == "'$EPAGES_INSTALL_WS_GID'") {print $1; exit 0}}' /etc/group`
fi
EPAGES_WEBGROUP=$EPAGES_INSTALL_WS_GROUP:$EPAGES_INSTALL_WS_GID

EPAGES_INSTALL_EP_GID=`awk -F: '/^epages:/{print $3; exit 0}' /etc/group`
EPAGES_INSTALL_AS_UID=`awk -F: '/^ep_appl:/{print $3; exit 0}' /etc/passwd`
EPAGES_INSTALL_DB_UID=`awk -F: '/^ep_db:/{print $3; exit 0}' /etc/passwd`
EPAGES_INSTALL_WS_UID=`awk -F: '/^ep_web:/{print $3; exit 0}' /etc/passwd`

EPAGES_INSTALL_WS_RUNUSER=`repair_account "$EPAGES_INSTALL_WS_RUNUSER"`
export EPAGES_INSTALL_WS_RUNUSER=${EPAGES_INSTALL_WS_RUNUSER:-eprunweb}
uid=`awk -F: '{if ($1 == ENVIRON["EPAGES_INSTALL_WS_RUNUSER"]) {print $3; exit 0}}' /etc/passwd`
EPAGES_INSTALL_WS_RUNUID=${uid:-$EPAGES_INSTALL_WS_RUNUID}

EPAGES_INSTALL_AS_RUNUSER=`repair_account "$EPAGES_INSTALL_AS_RUNUSER"`
export EPAGES_INSTALL_AS_RUNUSER=${EPAGES_INSTALL_AS_RUNUSER:-eprunapp}
uid=`awk -F: '{if ($1 == ENVIRON["EPAGES_INSTALL_AS_RUNUSER"]) {print $3; exit 0}}' /etc/passwd`
EPAGES_INSTALL_AS_RUNUID=${uid:-$EPAGES_INSTALL_AS_RUNUID}

#+++++++++++++++++++++++++++++#
#       GLOBAL OPTIONS        #
#+++++++++++++++++++++++++++++#

chgrp -L root / >/dev/null 2>&1 && CHGRP_L=-L || unset CHGRP_L
chown -L root / >/dev/null 2>&1 && CHOWN_L=-L || unset CHOWN_L

RPM_INSTALL_PREFIX=`cd "$RPM_INSTALL_PREFIX" && pwd`
EPAGES_TZ="\"$(local_timezone)\""

EPAGES_ACL=1
touch "$EPAGES"/acltest
setfacl -m u:ep_appl:rwx "$EPAGES"/acltest >/dev/null 2>&1 || unset EPAGES_ACL
rm -f "$EPAGES"/acltest

[ "$EPAGES_INSTALL_WS_HOST$EPAGES_INSTALL_AS_HOST$EPAGES_INSTALL_RR_HOST" = \
  "${EPAGES_INSTALL_WS_HOST//:/}${EPAGES_INSTALL_AS_HOST//:/}${EPAGES_INSTALL_RR_HOST//:/}" ] \
&& EPAGES_ENABLE_IPv6= || EPAGES_ENABLE_IPv6=1

if [ -s "$RPM_INSTALL_PREFIX"/etc/epages-project ] ; then
  EPAGES_PROJECT=`head -1 "$RPM_INSTALL_PREFIX"/etc/epages-project`
else
  EPAGES_PROJECT=`repair_account -d "$EPAGES_INSTALL_RUN_SCRIPT" epages6`
  if [ -d "$RPM_INSTALL_PREFIX"/etc ] ; then
    echo "$EPAGES_PROJECT" > "$RPM_INSTALL_PREFIX"/etc/epages-project
    chown ep_appl:epages "$RPM_INSTALL_PREFIX"/etc/epages-project
    chmod 644 "$RPM_INSTALL_PREFIX"/etc/epages-project
  fi
fi
EPAGES_INSTALL_RUN_SCRIPT=$EPAGES_PROJECT

is_positive "$EPAGES_INSTALL_MEMORY" || \
EPAGES_INSTALL_MEMORY=`awk '/^MemTotal/{print $2; exit 0}' < /proc/meminfo`

is_positive "$EPAGES_INSTALL_CPU_NUMBER" || \
EPAGES_INSTALL_CPU_NUMBER=`awk 'BEGIN{i=0}{if(/^processor/) i++}END{print i}' < /proc/cpuinfo`

MIN_ASERVER=2         # min. required number of app servers
MAX_AS_PER_CPU=4      # max. number of app servers per CPU
OS_MEMORY=250000      # memory used by operating  system KB
MIN_AS_MEMORY=75000   # memory used per application server
MIN_DB_MEMORY=250000  # memory used by database server (KB)

FREE_MEMORY=$(($EPAGES_INSTALL_MEMORY - $OS_MEMORY))

unset EPAGES_RUNNING_DAEMONS

PROCESS_service="$EPSERVICE"
PROCESS_rr="$EP_RR"
PROCESS_db="$SYBASE/$SYBASE_ASE/bin/dataserver"
PROCESS_dbmon="$SYBASE/$SYBASE_ASE/bin/monserver"
PROCESS_sysam="$SYBASE/$SYBASE_SYSAM/bin/lmgrd"

for i in service rr httpd db dbmon sysam ; do
  PROCESS="`eval echo '$'PROCESS_$i`"
  ps -e -oargs= | awk '{if ($1 == "'"$PROCESS"'") exit 1;}' || EPAGES_RUNNING_DAEMONS="$EPAGES_RUNNING_DAEMONS $i"
done

if [ "$EPAGES_RUNNING_DAEMONS" = "${EPAGES_RUNNING_DAEMONS//httpd}" ] ; then
  PROCESS="$HTTPD_ROOT/bin/httpd-epages"
 ps -e -oargs= | awk '{if ($1 == "'"$PROCESS"'") exit 1;}' || EPAGES_RUNNING_DAEMONS="$EPAGES_RUNNING_DAEMONS httpd"
fi

#+++++++++++++++++++++++++++++#
# APPLICATION SERVER OPTIONS  #
#+++++++++++++++++++++++++++++#

dns_lookup AS
dns_lookup DB

# if AS and DB are joined, use half free memory
FREE_AS_MEMORY=$(($FREE_MEMORY / 2))
# if AS and DB are separated, use all free memory
[ "${EPAGES_INSTALL_AS_ADDR}" != "${EPAGES_INSTALL_DB_ADDR}" ] && FREE_AS_MEMORY=$FREE_MEMORY

EPAGES_INSTALL_AS_FIRST_PORT=`repair_16bit -u "$EPAGES_INSTALL_AS_FIRST_PORT" 10045`

is_positive "$EPAGES_INSTALL_AS_NUMBER" || {
  EPAGES_INSTALL_AS_NUMBER=$(($FREE_AS_MEMORY / $MIN_AS_MEMORY))
  # if there are more than 4 AS per CPU, reduce the number
  MAX_AS_NUMBER=$(($EPAGES_INSTALL_CPU_NUMBER * $MAX_AS_PER_CPU))
  [ $EPAGES_INSTALL_AS_NUMBER -gt $MAX_AS_NUMBER ] && EPAGES_INSTALL_AS_NUMBER=$MAX_AS_NUMBER
}

EPAGES_MAX_LICENSED_AS=`awk -F\" '{if ($2 == "AppServerProcesses" && $6 ~ /^[0-9]*$/) {print $6; exit 0}}' "$EPAGES_CONFIG"/License.xml 2>/dev/null`
EPAGES_MAX_LICENSED_AS=${EPAGES_MAX_LICENSED_AS:-4}

# use at least 2 application servers
[ $EPAGES_INSTALL_AS_NUMBER -lt $MIN_ASERVER ] && EPAGES_INSTALL_AS_NUMBER=$MIN_ASERVER
# use not more application servers than licensed
[ $EPAGES_INSTALL_AS_NUMBER -gt $EPAGES_MAX_LICENSED_AS ] && EPAGES_INSTALL_AS_NUMBER=$EPAGES_MAX_LICENSED_AS

#+++++++++++++++++++++++++++++#
#   REQUEST ROUTER OPTIONS    #
#+++++++++++++++++++++++++++++#

dns_lookup RR

EPAGES_INSTALL_RR_PORT=`repair_16bit -u "$EPAGES_INSTALL_RR_PORT" 10043`
EPAGES_INSTALL_RR_MON_PORT=`repair_16bit -u "$EPAGES_INSTALL_RR_MON_PORT" 10041`
EPAGES_INSTALL_RR_MC_PORT=`repair_16bit -u "$EPAGES_INSTALL_RR_MC_PORT" 10042`

#+++++++++++++++++++++++++++++#
#   DATABASE SERVER OPTIONS   #
#+++++++++++++++++++++++++++++#

# if AS and DB are joined, use remaining free memory
INSTALL_AS_NUMBER=$EPAGES_INSTALL_AS_NUMBER
# if AS and DB are separated, use all free memory
[ "${EPAGES_INSTALL_AS_ADDR}" != "${EPAGES_INSTALL_DB_ADDR}" ] && INSTALL_AS_NUMBER=0

EPAGES_INSTALL_DB_PORT=`repair_16bit -u "$EPAGES_INSTALL_DB_PORT" 2510`
EPAGES_INSTALL_DB_BS_PORT=`repair_16bit -u "$EPAGES_INSTALL_DB_BS_PORT" 2511`
EPAGES_INSTALL_DB_MS_PORT=`repair_16bit -u "$EPAGES_INSTALL_DB_MS_PORT" 2512`
EPAGES_INSTALL_DB_XP_PORT=`repair_16bit -u "$EPAGES_INSTALL_DB_XP_PORT" 2513`

EPAGES_DB_MEMORY=$(($FREE_MEMORY - ($INSTALL_AS_NUMBER * $MIN_AS_MEMORY) ))

# use at least 250000 KB memory for the database server
[ $EPAGES_DB_MEMORY -lt $MIN_DB_MEMORY ] && EPAGES_DB_MEMORY=$MIN_DB_MEMORY

EPAGES_INSTALLATION_LEVEL=1
[ $EPAGES_DB_MEMORY -ge  500000 ] && EPAGES_INSTALLATION_LEVEL=2
[ $EPAGES_DB_MEMORY -ge 1000000 ] && EPAGES_INSTALLATION_LEVEL=3

# use half of the CPUs for the database server
EPAGES_DB_CPUS=1
[ $EPAGES_INSTALL_CPU_NUMBER -ge 3 ] && let EPAGES_DB_CPUS="$EPAGES_INSTALL_CPU_NUMBER / 2 + $EPAGES_INSTALL_CPU_NUMBER % 2"
# additionally use the CPUs the application server doesn't need
let UNUSED_ASERVER="(($EPAGES_INSTALL_CPU_NUMBER - $EPAGES_DB_CPUS) * $MAX_AS_PER_CPU ) - $EPAGES_INSTALL_AS_NUMBER"
[ $UNUSED_ASERVER -gt 0 ] && let EPAGES_DB_CPUS+="$UNUSED_ASERVER / $MAX_AS_PER_CPU"

#+++++++++++++++++++++++++++++#
#     WEB SERVER OPTIONS      #
#+++++++++++++++++++++++++++++#

dns_lookup WS

EPAGES_INSTALL_WS_PORT=`repair_16bit "$EPAGES_INSTALL_WS_PORT" 80`
EPAGES_INSTALL_WS_SSL_PORT=`repair_16bit "$EPAGES_INSTALL_WS_SSL_PORT" 443`
EPAGES_INSTALL_WS_LOCAL_PORT=`repair_16bit "$EPAGES_INSTALL_WS_LOCAL_PORT" 8008`
EPAGES_INSTALL_WS_LOCAL_SSLPORT=`repair_16bit "$EPAGES_INSTALL_WS_LOCAL_SSLPORT" 8443`

#+++++++++++++++++++++++++++++#
#       SET ENVIRONMENT       #
#+++++++++++++++++++++++++++++#

if [ "${RPM_INSTALL_PREFIX:0:1}" = / ] ; then
  i=$RPM_INSTALL_PREFIX
  # chmod o+rx for all ancestors of eproot
  while [ -n "$i" ] ; do chmod o+rx $i 2>/dev/null ; i=${i%/*} ; done
fi

#------------------------------
FILE="$RPM_INSTALL_PREFIX"/etc/default/epages6
[ -f "$FILE".ept -o -f "$FILE" ] || FILE=/etc/default/${EPAGES_PROJECT:-epages6}
#------------------------------

backup_file "$FILE".ept
EPAGES_APPUSER=$EPAGES_INSTALL_AS_RUNUSER
EPAGES_WEBUSER=$EPAGES_INSTALL_WS_RUNUSER

for i in EPAGES_PROJECT EPAGES EPAGES_APPUSER EPAGES_WEBUSER EPAGES_WEBGROUP ; do
  CURRENT=$("$INI_CONFIG" -f "$FILE" -g -k $i 2>/dev/null)
  [[ -n $CURRENT ]] && continue
  "$INI_CONFIG" -f "$FILE" -k $i -v "`eval echo '$'$i`" -i
done

if [ -z "$UPDATE_APACHE" -a -z "$UPDATE_SYBASE" ] ; then
  
  for i in EPAGES_ENABLE_IPv6 EPAGES_TZ EPAGES_ACL EPAGES_IGNORE_SITE EPAGES_IGNORE_DEMOSHOP ; do
    "$INI_CONFIG" -f "$FILE" -k $i -v "`eval echo '$'${i//IGNORE/INSTALL_IGNORE}`" -i
  done
 if [ -n "$EPAGES_INSTALL_HTTPD_OPTS" ] ; then
    "$INI_CONFIG" -f "$FILE" -k HTTPD_OPTS -v "\"$EPAGES_INSTALL_HTTPD_OPTS\"" -i
  fi
fi

. "$FILE"

set -a # export all variables

[ -d "$RPM_INSTALL_PREFIX"/etc/default ] && prefix="$RPM_INSTALL_PREFIX" || unset prefix

for i in RPM_INSTALL_PREFIX `echo ${!EPAGES*}` ; do
  eval echo $i='$'$i
done > "$prefix"/etc/default/epages-installed.conf.~$EPOCH~
chown ep_appl "$prefix"/etc/default/epages-installed.conf.~$EPOCH~

###############################################################################
# INSTALL FUNCTIONS                                                           #
###############################################################################

#+++++++++++++++++++++++++++++#
#      COMMON FUNCTIONS       #
#+++++++++++++++++++++++++++++#

set_yumconf () {
  if [ "$RPM_INSTALL_PREFIX" != /srv/epages ] ; then
    if [ ! -d /srv/epages/eproot/bin ] ; then
      rmdir /srv/epages/* 2>/dev/null
      rm -f /srv/epages/* 2>/dev/null
      rmdir /srv/epages 2>/dev/null
      rm -f /srv/epages 2>/dev/null
      mkdir -m 755 -p /srv
      if [ -e /srv/epages -a ! -e /srv/epages/eproot ] ; then
        ( cd /srv/epages ; ln -s "$RPM_INSTALL_PREFIX"/eproot )
      elif [ ! -e /srv/epages ] ; then
        ( cd /srv ; ln -s "$RPM_INSTALL_PREFIX" epages )
      fi
    fi
  fi

  if [ -n "$1" ] ; then
    for j in $(ls -d /etc/yum.repos.d/*epages* 2>/dev/null) ; do
      sed -i "s,/RPMS/6\.[0-9]\+\([^0-9.]\),/RPMS/$1\1," $j
    done
  fi
}

set_config () {
  #-------------------------------
  local FILE=$EPAGES_CONFIG
  #-------------------------------
  cd "$EPAGES_CONFIG"
  for i in `find . -type f -name '*.initial'` ; do
    [ ! -f "${i%.initial}" ] && cp -p "$i" "${i%.initial}" || {
      [ "${i##*/}" = version.txt.initial ] && cat "$i" >> version.txt
    }
  done
}

set_etc_hosts () {
  #-------------------------------
  local FILE=/etc/hosts
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "set_etc_hosts" 1>&2
  DNS=("$EPAGES_INSTALL_AS_IP" "$EPAGES_INSTALL_AS_FQDN" "$EPAGES_INSTALL_RR_IP" "$EPAGES_INSTALL_RR_FQDN"\
       "$EPAGES_INSTALL_DB_IP" "$EPAGES_INSTALL_DB_FQDN" "$EPAGES_INSTALL_WS_IP" "$EPAGES_INSTALL_WS_FQDN")
  backup_file $FILE
  while read LINE ; do
    set -- `echo $LINE`
    IP=$1 ; shift ; HOST=$@
    WRITE_LINE=1
    unset ADDHOST RMHOST
    i=1

    while [ $i -lt ${#DNS[*]} ] ; do
      if [ -n "${DNS[$i-1]}" -a -n "${DNS[$i]}" ] ; then
        if [[ "$IP" == "${DNS[$i-1]}" ]] ; then
          [[ " $HOST " ==  *" ${DNS[$i]} "* || " $HOST " ==  *" ${DNS[$i]%%.*} "* ]] || {
            [ "${DNS[$i]}" = "${DNS[$i]%%.*}" ] && ADDHOST="$ADDHOST ${DNS[$i]}" || \
            ADDHOST="${DNS[$i]} ${DNS[$i]%%.*} $ADDHOST"
          }
          unset WRITE_LINE
        elif [[ " $HOST " ==  *" ${DNS[$i]} "* || " $HOST " ==  *" ${DNS[$i]%%.*} "* ]] ; then
          [[ " $HOST " ==  *" ${DNS[$i]} "* ]] && RMHOST="$RMHOST ${DNS[$i]}"
          [[ " $HOST " ==  *" ${DNS[$i]%%.*} "* ]] && RMHOST="$RMHOST ${DNS[$i]%%.*}"
          unset WRITE_LINE
        fi
      fi
      let i+=2
    done
    [ -n "$WRITE_LINE" ] && { echo "$LINE"; continue; }

    HOST=" $HOST "
    for i in $RMHOST ; do HOST=${HOST// $i / } ; done
    HOST="$ADDHOST $HOST"
    expr "$HOST" : '.*[^[:space:]]' >/dev/null 2>&1 && echo -e "$IP\t"`echo $HOST`
  done <  $FILE.~$EPOCH~ > $FILE
}

set_db_interface () {
  [ ! -f "$SYBASE"/locales/locales.dat ] && return

  #-------------------------------
  local FILE="$SYBASE"/interfaces
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "set_db_interface \"$EPAGES_INSTALL_DB_NAME\" \"$EPAGES_INSTALL_DB_ADDR\"" 1>&2
  # get the DBServer name
  EPAGES_INSTALL_DB_NAME=`"$INI_CONFIG" -f "$EPAGES_CONFIG/Database.d/Site.conf" -s 'Connection' -k datasource -g 2>/dev/null | grep '[:;]server=' | sed 's/.*[:;]server=\([^;]*\).*/\1/'` \
  || unset EPAGES_INSTALL_DB_NAME
  export EPAGES_INSTALL_DB_NAME
  EPAGES_INSTALL_DB_NAME=${EPAGES_INSTALL_DB_NAME:-DBServer}

  backup_file "$FILE" "$SYBASE"/locales/locales.dat

  if [ -f "$FILE".~$EPOCH~ ] ; then
    # delete DBServer name sections in interfaces
    awk 'BEGIN{i=1}{if (/^'"$EPAGES_INSTALL_DB_NAME"'(_(BS|MS|XP))?[[:space:]]/) {i=0}
                    else if (i || /^[^[:space:]]/) {i=1; print;}}' \
    < "$FILE".~$EPOCH~ > "$FILE"
  fi

  local ADDR
  local HOST=$EPAGES_INSTALL_DB_HOST
  [ "${HOST/:/}" != "$HOST" ] && ADDR=$EPAGES_INSTALL_DB_ADDR || ADDR=$EPAGES_INSTALL_DB_IP
  
  for i in "" _BS _MS _XP ; do
    # add DBServer name sections to interfaces
    echo $EPAGES_INSTALL_DB_NAME$i 5 5
    echo -e "\tquery tcp ether $ADDR `eval echo '$'EPAGES_INSTALL_DB${i}_PORT`"
    echo -e "\tmaster tcp ether $ADDR `eval echo '$'EPAGES_INSTALL_DB${i}_PORT`"
    if [ -z "$i" -a -n "$EPAGES_INSTALL_DB_IPv4" -a "${HOST/:/}" != "$HOST" ] ; then
      echo -e "\tquery tcp ether $EPAGES_INSTALL_DB_IPv4 `eval echo '$'EPAGES_INSTALL_DB${i}_PORT`"
      echo -e "\tmaster tcp ether $EPAGES_INSTALL_DB_IPv4 `eval echo '$'EPAGES_INSTALL_DB${i}_PORT`"
    fi
  done >> "$FILE"
}

set_package_ini () {
  #-------------------------------
  local FILE
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "set_package_ini $@" 1>&2
  case $1 in
    sybsrv) FILE="$SYBASE/$SYBASE_ASE"/$1.ini ; USER=ep_db ;;
    *) FILE="$EPAGES_CONFIG"/$1.ini ; USER=ep_appl ;;
  esac
  touch "$FILE"
  chown $USER:epages "$FILE"
  chmod 644 "$FILE"
}

exec_daemons () {
  [ -n "$VERBOSE" ] && echo "exec_daemons $@ ($EPAGES_RUNNING_DAEMONS )" 1>&2
  local FORCE
  [ "x$1" = "x-f" ] && { FORCE=1; shift; }
  local LEVEL=$1
  shift
  for i in $@ ; do
    [[ -z "$FORCE" && " $EPAGES_RUNNING_DAEMONS " != *" $i "* ]] && continue
    RUNSCRIPT="$RPM_INSTALL_PREFIX"/etc/init.d/epages6
    [ -x "$RUNSCRIPT" ] || RUNSCRIPT=/etc/init.d/"$EPAGES_PROJECT"
    [ ! -x "$RUNSCRIPT" -a -n "$FORCE" ] && continue
    "$RUNSCRIPT" ${LEVEL}_$i >/dev/null 2>&1
  done
}

link_etc () {
  #-------------------------------
  local FILE="$RPM_INSTALL_PREFIX"/etc/init.d/epages6
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "link_etc \"$EPAGES_PROJECT\"" 1>&2

  backup_file "$FILE".ept
  sed "s,_EPAGES_PROJECT_,$EPAGES_PROJECT,g" "$FILE".ept > "$FILE"
  chown root "$FILE"

  rm -f /etc/init.d/$EPAGES_PROJECT /etc/default/$EPAGES_PROJECT

  for i in /etc/init.d /etc/default ; do (
    cd $i ; ln -s "$RPM_INSTALL_PREFIX"$i/epages6 $EPAGES_PROJECT
  ) ; done

  PERL5LIB= /sbin/chkconfig --del $EPAGES_PROJECT >/dev/null 2>&1
  PERL5LIB= /sbin/chkconfig --add $EPAGES_PROJECT >/dev/null 2>&1
}

link_uninstall_epages () {
  #-------------------------------
  local FILE="install-epages.sh"
  #-------------------------------
  rm -f $EPAGES/bin/un$FILE
  cd $EPAGES/bin && ln $FILE un$FILE
}

setfacl4ep_appl () {
  return
  [ -n "$VERBOSE" ] && echo "setfacl4ep_appl $@" 1>&2
  if [ -n "$EPAGES_ACL" ] ; then
    [ -d "$RPM_INSTALL_PREFIX"/etc/init.d ] && prefix="$RPM_INSTALL_PREFIX" || unset prefix
    setfacl -m u:ep_appl:rwx "$prefix"/etc/init.d/epages6{,su} >/dev/null 2>&1
    setfacl -dL -m u:ep_appl:rwx "$EPAGES_CONFIG/" 2>/dev/null
    setfacl -L -m u:ep_appl:rwx "$EPAGES_CONFIG/" 2>/dev/null
    setfacl -RdL -m u:ep_appl:rwx $@ 2>/dev/null
    setfacl -RL  -m u:ep_appl:rwx $@ 2>/dev/null
  fi
}

update_sic () {
  local HOSTNAME=`hostname --short`
  local TASKS MACHINE
  unset TASKS
  for i in $@ ; do
    case $i in
      cartridges) TASKS="$TASKS Task:Cartridges:HH Task:Cartridges:MM" ;;
      perl) TASKS="$TASKS Task:Perl:MM" ;;
      shared) TASKS="$TASKS Task:Shared:MM" ;;
    esac
  done
  for i in $TASKS ; do
    MACHINE=,`"$INI_CONFIG" -f "$EPAGES_SHARED/Tools/CheckIntegrity.conf" -s "$i" -k Machine -g 2>/dev/null`, \
    || unset MACHINE
    if [ "$MACHINE" != "${MACHINE//,$HOSTNAME,/}" ] ; then
        "$INI_CONFIG" -f "$FILE" -s "$i" -k Machine -v "${MACHINE#,}$HOSTNAME" -i 2>/dev/null
    fi
  done
}

#+++++++++++++++++++++++++++++#
#        PERL FUNCTIONS       #
#+++++++++++++++++++++++++++++#

set_perl_tree () {
  #-------------------------------
  local FILE="Config.pm Config_heavy.pl ../site_perl/ppm.xml CORE/config.h"
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "set_perl_tree \"$EPAGES_PERL\"" 1>&2
  cd "$EPAGES_PERL"/lib/linux || return 1
  for i in $FILE ; do
    backup_file $i.ept
    sed "s,=EPAGES_PERL=,$EPAGES_PERL,g" $i.ept > $i
  done

  if grep -q ' Perl/' perllocal.pod 2>/dev/null ; then
    cp -p perllocal.pod perllocal.pod.tmp
    sed "s, Perl/, $EPAGES_PERL/,g" perllocal.pod.tmp > perllocal.pod
    rm -f perllocal.pod.tmp
  fi

  cd "$EPAGES_PERL"/lib
  for i in $(find . -name .packlist -exec grep \-l '^Perl/' {} \;) ; do
    cp -p $i $i.tmp
    sed "s,^Perl/,$EPAGES_PERL/,g" $i.tmp > $i
    rm -f $i.tmp
  done
}

#+++++++++++++++++++++++++++++#
#      APPSRV FUNCTIONS       #
#+++++++++++++++++++++++++++++#

set_epages_conf () {
  #-------------------------------
  local FILE="$EPAGES_CONFIG"/epages.conf
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "set_epages_conf \"$EPAGES_INSTALL_WS_ADDR\"" 1>&2
  backup_file "$FILE".ept
  local CONN=$EPAGES_INSTALL_WS_ADDR
  [ "$CONN" != ${CONN/:/} ] && CONN="[$CONN]"
  [ "$EPAGES_INSTALL_WS_PORT" != 80 ]  && CONN="$CONN:$EPAGES_INSTALL_WS_PORT"
  "$INI_CONFIG" -f "$FILE" -s "DE_EPAGES::Object" -k SystemDomainName -v "$EPAGES_INSTALL_WS_ADDR" -i 2>/dev/null
  "$INI_CONFIG" -f "$FILE" -s "DE_EPAGES::Store" -k DefaultWebServiceProxy -r _SystemDomainName_ -v "$CONN" -i 2>/dev/null
}

set_webinterface_rr () {
  #-------------------------------
  local FILE="$EPAGES_CONFIG"/WebInterface.conf
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "set_webinterface_rr \"$EPAGES_INSTALL_RR_IP\"" 1>&2
  backup_file "$FILE".ept

  local WI_KEY WI_VALUE
  WI_KEY=("MONITOR" "MESSAGECENTER" "REQUESTROUTER")
  WI_VALUE=($EPAGES_INSTALL_RR_MON_PORT $EPAGES_INSTALL_RR_MC_PORT $EPAGES_INSTALL_RR_PORT)
  local QUOTED_IP=${EPAGES_INSTALL_RR_IP//\./\\.}

  for i in 0 1 2 ; do
    "$INI_CONFIG" -f "$FILE" -s "$QUOTED_IP" -k "${WI_KEY[$i]}" -v "${WI_VALUE[$i]}" -i 2>/dev/null
  done
  for i in $EPAGES_INSTALL_RR_NET ; do
    "$INI_CONFIG" -f "$FILE" -s "$i" -i 2>/dev/null
     [ -z "${i//[^:]/}" ] && "$INI_CONFIG" -f "$FILE" -s "::ffff:$i" -i 2>/dev/null
  done

  if [[ "$EPAGES_INSTALL_RR_DEFAULT" != *[0-9.a-zA-Z]* ]] ; then
    "$INI_CONFIG" -f /etc/sysconfig/epages6 -k EPAGES_INSTALL_RR_DEFAULT -v "$EPAGES_INSTALL_RR_IP" -i 2>/dev/null
    # if EPAGES_INSTALL_RR_DEDICATED isn't empty, add DEDICATED
    if [[ "$EPAGES_INSTALL_RR_DEDICATED" == *[0-9.a-zA-Z]* ]] ; then
      "$INI_CONFIG" -f "$FILE" -s "$QUOTED_IP" -k DEDICATED -v "$EPAGES_INSTALL_RR_DEDICATED" -i 2>/dev/null
    fi
  fi

  local EXCEPTION_HOST=" "
  for i in `"$INI_CONFIG" -f "$FILE" -s URLRewrite -k ExceptionHost -g 2>/dev/null` \
  $EPAGES_INSTALL_WS_IP $EPAGES_INSTALL_WS_IPv4 $EPAGES_INSTALL_WS_ADDR \
  $EPAGES_INSTALL_WS_FQDN ${EPAGES_INSTALL_WS_FQDN%%.*} ; do
    [ "${i%:*}" != "$i" -a "${i%[*}" = "$i" ] && i="[$i]"
    q="$(echo "$i" | sed 's/\([].[]\)/\\\1/'g)"
    [ "${EXCEPTION_HOST// $q /}" = "$EXCEPTION_HOST" ] || continue
    EXCEPTION_HOST="$EXCEPTION_HOST$i "
  done
  EXCEPTION_HOST="${EXCEPTION_HOST% }"
  "$INI_CONFIG" -f "$FILE" -s URLRewrite -k ExceptionHost -v "${EXCEPTION_HOST# }" -i 2>/dev/null

  if [ -n "$EPAGES_INSTALL_DEFINED_HOST" ] ; then
    unset SET_LOCAL_IPS
    for i in $(LANG=C host $(hostname) 2>/dev/null | awk '/ address /{sub(/.* address /, "");print;}')
    do
      LANG=C ifconfig | grep -q "[: ]$i[/ ]" || { SET_LOCAL_IPS=1; break; }
    done
    [ -n "$SET_LOCAL_IPS" ] && "$EPAGES"/bin/set-local-ips.sh
  fi
}

set_webinterface_as () {
  #-------------------------------
  FILE="$EPAGES_CONFIG"/WebInterface.conf
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "set_webinterface_as \"$EPAGES_INSTALL_AS_IP\"" 1>&2
  backup_file "$FILE".ept

  local QUOTED_IP=${EPAGES_INSTALL_AS_IP//\./\\.}
  local AS_PORTS
  AS_PORTS=`"$INI_CONFIG" -f "$FILE" -s "$QUOTED_IP" -k APPLICATIONSERVER -g 2>/dev/null` || unset AS_PORTS
  AS_PORTS=" ${AS_PORTS//,/ } "

  set -- `echo $AS_PORTS`
  local REQ_AS_NUMBER=$(($EPAGES_INSTALL_AS_NUMBER - $#))

  # add $AS_PORTS to APPLICATIONSERVER in section $EPAGES_INSTALL_AS_IP up to $REQ_AS_NUMBER (new minus existing)
  while [ $REQ_AS_NUMBER -gt 0 ] ; do
    while [[ $AS_PORTS == *" "$EPAGES_INSTALL_AS_FIRST_PORT" "* ]] ; do
      let EPAGES_INSTALL_AS_FIRST_PORT+=1
    done
    AS_PORTS="$AS_PORTS $EPAGES_INSTALL_AS_FIRST_PORT "
    let REQ_AS_NUMBER-=1
  done

  AS_PORTS=`echo $AS_PORTS`
  AS_PORTS=${AS_PORTS// /,}

  "$INI_CONFIG" -f "$FILE" -s "$QUOTED_IP" -k APPLICATIONSERVER -v "$AS_PORTS" -i 2>/dev/null
  for i in $EPAGES_INSTALL_AS_NET ; do
    "$INI_CONFIG" -f "$FILE" -s "$i" -i 2>/dev/null
     [ -z "${i//[^:]/}" ] && "$INI_CONFIG" -f "$FILE" -s "::ffff:$i" -i 2>/dev/null
  done

  if [[ "$EPAGES_INSTALL_RR_DEDICATED" != *[0-9.a-zA-Z]* ]] ; then
    EPAGES_INSTALL_RR_DEDICATED="$EPAGES_INSTALL_AS_IP"
    "$INI_CONFIG" -f /etc/sysconfig/epages6 -k EPAGES_INSTALL_RR_DEDICATED -v "$EPAGES_INSTALL_RR_DEDICATED" -i 2>/dev/null
  elif [[ ",$EPAGES_INSTALL_RR_DEDICATED," != *,$EPAGES_INSTALL_AS_IP,* ]] ; then
    EPAGES_INSTALL_RR_DEDICATED="$EPAGES_INSTALL_RR_DEDICATED,$EPAGES_INSTALL_AS_IP"
    "$INI_CONFIG" -f /etc/sysconfig/epages6 -k EPAGES_INSTALL_RR_DEDICATED -v "$EPAGES_INSTALL_RR_DEDICATED" -i 2>/dev/null
  fi

  # if EPAGES_INSTALL_RR_DEFAULT isn't empty, add DEDICATED
  if [[ "$EPAGES_INSTALL_RR_DEFAULT" == *[0-9.a-zA-Z]* ]] ; then
    QUOTED_IP=${EPAGES_INSTALL_RR_DEFAULT//\./\\.}
    "$INI_CONFIG" -f "$FILE" -s "$QUOTED_IP" -k DEDICATED -v "$EPAGES_INSTALL_RR_DEDICATED" -i 2>/dev/null
  fi

  local EXCEPTION_HOST=" "
  for i in `"$INI_CONFIG" -f "$FILE" -s URLRewrite -k ExceptionHost -g 2>/dev/null` \
  $EPAGES_INSTALL_WS_IP $EPAGES_INSTALL_WS_IPv4 $EPAGES_INSTALL_WS_ADDR \
  $EPAGES_INSTALL_WS_FQDN ${EPAGES_INSTALL_WS_FQDN%%.*} ; do
    [ "${i%:*}" != "$i" -a "${i%[*}" = "$i" ] && i="[$i]"
    q="$(echo "$i" | sed 's/\([].[]\)/\\\1/'g)"
    [ "${EXCEPTION_HOST// $q /}" = "$EXCEPTION_HOST" ] || continue
    EXCEPTION_HOST="$EXCEPTION_HOST$i "
  done
  EXCEPTION_HOST="${EXCEPTION_HOST% }"
  "$INI_CONFIG" -f "$FILE" -s URLRewrite -k ExceptionHost -v "${EXCEPTION_HOST# }" -i 2>/dev/null

  if [ -n "$EPAGES_INSTALL_DEFINED_HOST" ] ; then
    unset SET_LOCAL_IPS
    for i in $(LANG=C host $(hostname) 2>/dev/null | awk '/ address /{sub(/.* address /, "");print;}')
    do
      LANG=C ifconfig | grep -q "[: ]$i[/ ]" || { SET_LOCAL_IPS=1; break; }
    done
    [ -n "$SET_LOCAL_IPS" ] && "$EPAGES"/bin/set-local-ips.sh
  fi

  if [[ ( -z $EPAGES_MYSQL_CONNECTION || ${EPAGES_MYSQL_CONNECTION#*/} != $EPAGES_MYSQL_CONNECTION )
        && -n $EPAGES_INSTALL_MYSQL_HOST && -n $EPAGES_INSTALL_MYSQL_TCPCONN
        && $(hostname) != $EPAGES_INSTALL_MYSQL_HOST ]] ; then
    FILE="$RPM_INSTALL_PREFIX"/etc/default/epages6
    "$INI_CONFIG" -f $FILE -k EPAGES_MYSQL_CONNECTION -v "\"$EPAGES_INSTALL_MYSQL_TCPCONN\"" -i
  fi
}

set_webinterface_ws () {
  #-------------------------------
  local FILE="$EPAGES_CONFIG"/WebInterface.conf
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "set_webinterface_ws \"$EPAGES_INSTALL_WS_IP\"" 1>&2
  backup_file "$FILE".ept

  local EXCEPTION_HOST=" "
  for i in `"$INI_CONFIG" -f "$FILE" -s URLRewrite -k ExceptionHost -g 2>/dev/null` \
  $EPAGES_INSTALL_WS_IP $EPAGES_INSTALL_WS_IPv4 $EPAGES_INSTALL_WS_ADDR \
  $EPAGES_INSTALL_WS_FQDN ${EPAGES_INSTALL_WS_FQDN%%.*} ; do
    [ "${i%:*}" != "$i" -a "${i%[*}" = "$i" ] && i="[$i]"
    q="$(echo "$i" | sed 's/\([].[]\)/\\\1/'g)"
    [ "${EXCEPTION_HOST// $q /}" = "$EXCEPTION_HOST" ] || continue
    EXCEPTION_HOST="$EXCEPTION_HOST$i "
  done
  EXCEPTION_HOST="${EXCEPTION_HOST% }"
  "$INI_CONFIG" -f "$FILE" -s URLRewrite -k ExceptionHost -v "${EXCEPTION_HOST# }" -i 2>/dev/null
}

change_app_user () {
  [ "$1" = "-R" ] && { R=" -R"; shift; } || unset R
  [ -n "$VERBOSE" ] && echo "change_app_user$R $EPAGES_INSTALL_AS_RUNUSER" 1>&2
  chown -f $R $CHOWN_L $EPAGES_INSTALL_AS_RUNUSER $@
}

add_mysql_group () {
  # add ep_appl and eprunapp to group mysql
  for i in ep_appl $EPAGES_INSTALL_AS_RUNUSER ; do
    grep -q '^mysql:' /etc/group || continue
    MEMBERS=$(awk -F: '{if ($1 == "mysql") {print $4; exit 0}}' /etc/group)
    MEMBERS=${MEMBERS:+$MEMBERS,}
    if [[ ",$MEMBERS" !=  *",$i,"* ]] ; then
      [ -n "$VERBOSE" ] && echo "add_mysql_group: $i -> mysql"
      cp -p /etc/group /etc/group.~$$~
      sed "s/^\(mysql:.*:\).*/\1$MEMBERS$i/" /etc/group.~$$~ > /etc/group
    fi
  done
  rm -f /etc/group.~$$~

  # add mysql to group epages
  grep -q '^epages:' /etc/group || return
  MEMBERS=$(awk -F: '{if ($1 == "epages") {print $4; exit 0}}' /etc/group)
  MEMBERS=${MEMBERS:+$MEMBERS,}
  if [[ ",$MEMBERS" !=  *",mysql,"* ]] ; then
    [ -n "$VERBOSE" ] && echo "add_mysql_group: mysql -> epages"
    cp -p /etc/group /etc/group.~$$~ || exit 1
    sed "s/^\(epages:.*:\).*/\1${MEMBERS}mysql/" /etc/group.~$$~ > /etc/group || exit 1
  fi
  rm -f /etc/group.~$$~
}

#+++++++++++++++++++++++++++++#
#      SYBSRV FUNCTIONS       #
#+++++++++++++++++++++++++++++#

set_dbserver_cfg () {
  #-------------------------------
  local FILE="$SYBASE/$SYBASE_ASE"/DBServer.cfg
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "set_dbserver_cfg $EPAGES_INSTALLATION_LEVEL $EPAGES_DB_MEMORY $EPAGES_DB_CPUS" 1>&2
  backup_file "$FILE".ept$EPAGES_INSTALLATION_LEVEL
  # edit $FILE and execute sysctl
  "$SYBASE"/bin/db-memory.sh -k "$EPAGES_DB_MEMORY" -# "$EPAGES_DB_CPUS" "$FILE" >/dev/null 2>&1

  find "$SYBASE/$SYBASE_ASE"/Scheduler.d -type f -name '*.initial' | while read i ; do
    [ -f "${i%.initial}" ] || cp -p "$i" "${i%.initial}"
  done
}

set_run_dbserver () {
  #-------------------------------
  local FILE="$SYBASE/$SYBASE_ASE"/install/RUN_DBServer
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "set_run_dbserver \"$SYBASE\"" 1>&2
  for i in "" _BS _MS ; do
    backup_file "$FILE"$i.ept
    "$INI_CONFIG" -f "$FILE"$i -k "SYBASE" -v "$SYBASE" -i
  done
}

set_vpd_properties () {
  #-------------------------------
  local FILE="$SYBASE"/vpd.properties
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "set_vpd_properties \"$FILE\"" 1>&2
  [ "$(/bin/uname -p)" = x86_64 ] && ARCH=64 || ARCH=32
  backup_file "$FILE".ept$ARCH
  sed "s,=SYBASE=,$SYBASE,g" "$FILE".ept$ARCH > "$FILE"
  ( cd "$SYBASE"/UAF-2_0 ; rm -f rtlib
    mv rtlib rtlib.obsolete ; ln -s rtlib$ARCH rtlib ) 2>/dev/null
}

set_agent_plugin () {
  #-------------------------------
  local FILE="$SYBASE"/UAF-2_0/plugins/com.sybase.ase/agent-plugin.xml
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "set_agent_plugin \"$SYBASE\"" 1>&2
  backup_file "$FILE".ept
  sed "s,=SYBASE=,$SYBASE,g" "$FILE".ept > "$FILE"
}

set_srv_res () {
  #-------------------------------
  local FILE="$SYBASE/$SYBASE_ASE"
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "set_srv_res \"$FILE\"" 1>&2

  local SRV_KEY SRV_PORT
  SRV_KEY=(sqlsrv bsrv msrv xp)
  SRV_PORT=($EPAGES_INSTALL_DB_PORT $EPAGES_INSTALL_DB_BS_PORT $EPAGES_INSTALL_DB_MS_PORT $EPAGES_INSTALL_DB_XP_PORT)

  for i in 0 1 2 3 ; do
    backup_file "$FILE"/${SRV_KEY[$i]}.res.ept
    sed "s,_SYBASE_,$SYBASE,;s,_HOST_,$EPAGES_INSTALL_DB_ADDR,;s,_PORT_,${SRV_PORT[$i]}," \
      "$FILE"/${SRV_KEY[$i]}.res.ept > "$FILE"/${SRV_KEY[$i]}.res
  done
}

set_initial_dat () {
  #-------------------------------
  local FILE="$SYBASE/data"
  #-------------------------------
  cd "$FILE" 2>/dev/null || return 1
  [ -n "$VERBOSE" ] && echo "set_initial_dat \"$FILE\"" 1>&2
  for i in $(ls *.initial) ; do mv $i ${i%.initial} ; done
}

link_jre () {
  #-------------------------------
  local FILE="$SYBASE/shared/sybcentral600/jre"
  #-------------------------------
  cd "${FILE%/*}" 2>/dev/null || return 1
  [ -n "$VERBOSE" ] && echo "link_jre \"$FILE\"" 1>&2
  rm -f jre 2>/dev/null
  [ "$(/bin/uname -p)" = x86_64 ] && ARCH=64 || ARCH=32
  rm -f jre
  mv jre jre.obsolete 2>/dev/null
  ln -s ../JRE-6_0_6_${ARCH}BIT jre
}

register_asep () {
  #-------------------------------
  local SCROOT="$SYBASE"/shared/sybcentral600
  local FILE="$SCROOT"/.scRepository600
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "register_asep \"$FILE\"" 1>&2
  cp -p "$FILE" "$FILE"_backup 2>/dev/null && \
  grep -v Providers/ASE "$FILE"_backup > "$FILE" 2>/dev/null
  "$SYBASE"/ASEP/bin/registerASEP >/dev/null 2>&1

  FILE="$SYBASE"/AMCP/amcplugin.jpr
  cp -p "$FILE".ept "$FILE"
  sed "s,=SYBASE=,$SYBASE,g" "$FILE".ept > "$FILE"
}

register_sysam () {
  #-------------------------------
  local SCROOT="$SYBASE"/shared/sybcentral600
  local FILE="$SCROOT"/.scRepository600
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "register_sysam \"$FILE\"" 1>&2
  cp -p "$FILE" "$FILE"_backup 2>/dev/null && \
  grep -v Providers/SYSAM "$FILE"_backup > "$FILE" 2>/dev/null
  "$SYBASE"/SYSAM-2_0/SYSAMP/bin/registerSYSAMP >/dev/null 2>&1
}

#+++++++++++++++++++++++++++++#
#      WEBSRV FUNCTIONS       #
#+++++++++++++++++++++++++++++#

set_httpd_conf () {
  #-------------------------------
  local FILE="$HTTPD_ROOT"/conf/httpd.conf
  #-------------------------------

  case `uname -r` in
    2.6.16[.-]*) KERNEL=2.6.16s ;;
    2.6.18[.-]*) KERNEL=2.6.18r ;;
    2.6.27[.-]*) KERNEL=2.6.27s ;;
    2.6.32-*)    KERNEL=2.6.32r ;;
    2.6.32.*)    KERNEL=2.6.32s ;;
  esac

  EPAGES_REWRITE_CONF=${EPAGES_REWRITE_CONF:-"$RPM_INSTALL_PREFIX"/eproot/Apache/conf/extra/zzz-epages-rewrite.conf}
  unset SERVER_ALIAS
  if [[ $EPAGES_INSTALL_WS_ADDR != ${EPAGES_INSTALL_WS_FQDN%%.*} ]] ; then
    SERVER_ALIAS="ServerAlias ${EPAGES_INSTALL_WS_FQDN%%.*}"
  fi
  if [[ $EPAGES_INSTALL_WS_ADDR != $EPAGES_INSTALL_WS_FQDN && $EPAGES_INSTALL_WS_FQDN != ${EPAGES_INSTALL_WS_FQDN%%.*} ]] ; then
    SERVER_ALIAS="$SERVER_ALIAS\nServerAlias $EPAGES_INSTALL_WS_FQDN"
  fi

  [ -n "$VERBOSE" ] && echo "set_httpd_conf \"$HTTPD_ROOT/conf\"" 1>&2
  backup_file "$FILE".ept
  sed "
  s,_CONF_DIR_,$HTTPD_ROOT,g;
  s,_EPAGES_PERL_,$EPAGES_PERL,g;
  s,_EPAGES_,$EPAGES,g;
  s,_SERVERALIAS_,$SERVER_ALIAS,g;
  s,_HOST_,$EPAGES_INSTALL_WS_ADDR,g;
  s,_GROUP_,$EPAGES_INSTALL_WS_GROUP,g;
  s,_USER_,$EPAGES_INSTALL_WS_RUNUSER,g;
  s,_PORT_,$EPAGES_INSTALL_WS_PORT,g;
  s,_SSLPORT_,$EPAGES_INSTALL_WS_SSL_PORT,g;
  s,_LOCALPORT_,$EPAGES_INSTALL_WS_LOCAL_PORT,g;
  s,_LOCALSSLPORT_,$EPAGES_INSTALL_WS_LOCAL_SSLPORT,g;
  s,_REWRITE_CONF_,$EPAGES_REWRITE_CONF,g;
  s,_IP_,$EPAGES_INSTALL_WS_IP,g;
  s,_KERNEL_,$KERNEL,g;
  s,_LIB_,,g;
  s,_MODULES_,modules,g;
  " "$FILE".ept > "$FILE"

  for FILE in $(ls -d "$EPAGES"/WebAdapter/*.ept "$HTTPD_ROOT"/conf/extra/*.ept 2>/dev/null) ; do
    backup_file "$HTTPD_ROOT"/conf/extra/`basename "$FILE" .ept` "$HTTPD_ROOT"/conf/httpd.conf.ept
    sed "
    s,_CONF_DIR_,$HTTPD_ROOT,g;
    s,_EPAGES_PERL_,$EPAGES_PERL,g;
    s,_EPAGES_,$EPAGES,g;
    s,_SERVERALIAS_,$SERVER_ALIAS,g;
    s,_HOST_,$EPAGES_INSTALL_WS_ADDR,g;
    s,_GROUP_,$EPAGES_INSTALL_WS_GROUP,g;
    s,_USER_,$EPAGES_INSTALL_WS_RUNUSER,g;
    s,_PORT_,$EPAGES_INSTALL_WS_PORT,g;
    s,_SSLPORT_,$EPAGES_INSTALL_WS_SSL_PORT,g;
    s,_LOCALPORT_,$EPAGES_INSTALL_WS_LOCAL_PORT,g;
    s,_LOCALSSLPORT_,$EPAGES_INSTALL_WS_LOCAL_SSLPORT,g;
    s,_REWRITE_CONF_,$EPAGES_REWRITE_CONF,g;
    s,_IP_,$EPAGES_INSTALL_WS_IP,g;
    s,_KERNEL_,$KERNEL,g;
    s,_LIB_,,g;
    s,_MODULES_,modules,g;
    " "$FILE" > "$HTTPD_ROOT"/conf/extra/`basename "$FILE" .ept`
  done

  FILE="$EPAGES"/WebAdapter/zzz-epages-lb.local
  backup_file "$FILE" "$HTTPD_ROOT"/conf/httpd.conf.ept

  sed "
  s,_EPAGES_,$EPAGES,g;
  s,_CONF_DIR_,$CONF_DIR,g;
  s,_SERVERALIAS_,$SERVER_ALIAS,g;
  s,_HOST_,$EPAGES_INSTALL_WS_ADDR,g;
  s,_IP_,$EPAGES_INSTALL_WS_IP,g;
  s,_REWRITE_CONF_,$EPAGES_REWRITE_CONF,g;
  " "$FILE" > "$HTTPD_ROOT"/conf/extra/`basename "$FILE"`

  find "$HTTPD_ROOT"/conf -type f -name '*.initial' | while read i ; do
    [ -f "${i%.initial}" ] || cp -p "$i" "${i%.initial}"
  done

  find "$HTTPD_ROOT"/conf -name '*.epx' | while read i ; do
    backup_file "$i"
    sed "s,_EPAGES_,$EPAGES,g" "$i" > "${i%.epx}"
  done

  "$EPAGES"/bin/update-rewrite-conf.sh
}

set_httpd_root () {
  #-------------------------------
  local FILE="$HTTPD_ROOT"/bin
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "set_httpd_root \"$HTTPD_ROOT\"" 1>&2
  if cd "$FILE" ; then
    for i in  *.ept ; do
      backup_file "$i"
      sed "s,=HTTPD_ROOT=,$HTTPD_ROOT," "$i" > "${i%.ept}"
    done
    [ -e httpd-epages ] && rm -f httpd-epages
    ln -s httpd httpd-epages
  fi

  find "$HTTPD_ROOT" -name '*.epx' ! -name '*.conf.epx' | while read i ; do
    backup_file "$i"
    sed "s,_EPAGES_,$EPAGES,g" "$i" > "${i%.epx}"
  done
}

change_web_group () {
  [ -n "$VERBOSE" ] && echo "change_web_group $EPAGES_INSTALL_WS_GROUP" 1>&2
  chgrp -fR $CHGRP_L $EPAGES_INSTALL_WS_GROUP $@
  chown -fR $CHOWN_L $EPAGES_INSTALL_WS_RUNUSER "$HTTPD_ROOT"/cache 2>/dev/null
}

###############################################################################
# UNINSTALL FUNCTIONS                                                         #
###############################################################################

#+++++++++++++++++++++++++++++#
#      COMMON FUNCTIONS       #
#+++++++++++++++++++++++++++++#

reset_config () {
  return
  #-------------------------------
  local FILE=$EPAGES_CONFIG
  #-------------------------------
  cd "$EPAGES_CONFIG" 2>/dev/null || return 1
  for i in `find . -type f -name '*.initial'` ; do
    rm -f "${i%.initial}"
  done
}

reset_etc_hosts () {
  return
  #-------------------------------
  local FILE=/etc/hosts
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "reset_etc_hosts $FILE.~*~" 1>&2
  rm -f $FILE.~*~ /etc/group.~*~
}

reset_perl_tree () {
  return
  #-------------------------------
  local FILE="Config.pm Config_heavy.pl ../site_perl/ppm.xml CORE/config.h "
  #-------------------------------
  cd "$EPAGES_PERL"/lib/linux 2>/dev/null || return 1
  [ -n "$VERBOSE" ] && echo "reset_perl_tree $FILE ${FILE// /.~*~ }" 1>&2
  rm -f $FILE ${FILE// /.~*~ } perllocal.pod

  find "$EPAGES_PERL"/lib/ -name .packlist -exec rm \-f {} \;

}

reset_db_interface () {
  return
  #-------------------------------
  local FILE="$SYBASE"/interfaces
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "reset_db_interface \"$FILE\" \"$FILE\".~*~" 1>&2
  rm -f "$FILE".~*~
}

reset_package_ini () {
  return
  #-------------------------------
  local FILE
  #-------------------------------
  case $1 in
    sybsrv) FILE="$SYBASE/$SYBASE_ASE"/$1.ini ;;
    *) FILE="$EPAGES_CONFIG"/$1.ini ;;
  esac
  [ -n "$VERBOSE" ] && echo "reset_package_ini \"$FILE\"" 1>&2
  rm -f "$FILE"
}

unlink_etc () {
  [ -n "$VERBOSE" ] && echo "unlink_etc" 1>&2
  #------------------------------
  local FILE="$RPM_INSTALL_PREFIX"/etc/created-accounts
  #------------------------------
  sort -r "$FILE" 2>/dev/null | while read type account ; do
    [ -z "$account" ] && continue
    [ "$type" = user ] && OPT=-r || unset OPT
    /usr/sbin/"$type"del $OPT "$account" >/dev/null 2>&1
  done

  #------------------------------
  FILE="$RPM_INSTALL_PREFIX"/etc
  #------------------------------
  PERL5LIB= /sbin/chkconfig --del $EPAGES_PROJECT >/dev/null 2>&1
  rm -f /etc/init.d/$EPAGES_PROJECT.~*~ /etc/default/$EPAGES_PROJECT.~*~
  rm -f "$FILE"/{epages-project,created-accounts}
  [ -x /sbin/insserv ] && /sbin/insserv 2>/dev/null
  rm -f "$FILE"/{default,init.d}/epages6.~*~
  rm -f "$FILE"/default/epages-installed.conf.~*~
}

clean_log_dir () {
  [ -n "$VERBOSE" ] && echo "clean_log_dir" 1>&2
  case $1 in
    AS) rm -rf "$EPAGES_LOG"/* ;;
    DB) rm -rf "$SYBASE_ASE_LOG"/* "$EPAGES"/Setup/*.sql.log ;;
    WS) rm -rf "$HTTPD_ROOT"/logs/* ;;
  esac
}

#+++++++++++++++++++++++++++++#
#      APPSRV FUNCTIONS       #
#+++++++++++++++++++++++++++++#

reset_epages_xml () {
  return
  #-------------------------------
  local FILE="Object/Database/XML/System.xml ShopConfiguration/Database/XML/Stores.xml "
  #-------------------------------
  cd "$RPM_INSTALL_PREFIX"/eproot/Cartridges/DE_EPAGES 2>/dev/null || return 1
  [ -n "$VERBOSE" ] && echo "reset_epages_xml $FILE ${FILE// /.~*~ }" 1>&2
  rm -f $FILE ${FILE// /.~*~ }
}

reset_epages_conf () {
  return
  #-------------------------------
  local FILE="$EPAGES_CONFIG"/epages.conf
  #-------------------------------
  rm -f "$FILE".~*~
  [ -n "$VERBOSE" ] && echo "reset_epages_conf \"$FILE\".~*~" 1>&2
}

reset_webinterface_conf () {
  return
  #-------------------------------
  local FILE="$EPAGES_CONFIG"/WebInterface.conf
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "reset_webinterface_conf \"$FILE\" \"$FILE\".~*~" 1>&2
  rm -f "$FILE".~*~ "$EPAGES_CONFIG"/{ASPool.db,EbayCategories.db,Cartridges.conf}
}

reset_local_httpd () {
  return
  [ -n "$VERBOSE" ] && echo "reset_local_httpd ($EPAGES/bin/use-local-httpd.sh -u)" 1>&2
  "$EPAGES"/bin/use-local-httpd.sh -u
}


#+++++++++++++++++++++++++++++#
#      SYBSRV FUNCTIONS       #
#+++++++++++++++++++++++++++++#

reset_dbserver_cfg () {
  return
  #-------------------------------
  local FILE="$SYBASE/$SYBASE_ASE"/DBServer.cfg
  #-------------------------------
  rm -f "$FILE".~*~  "$SYBASE/$SYBASE_ASE"/*.{bak,krg}
  if [ -f /etc/sysctl.conf.preepages ] ; then
    cat /etc/sysctl.conf.preepages > /etc/sysctl.conf
    touch -r /etc/sysctl.conf.preepages /etc/sysctl.conf
    rm -f /etc/sysctl.conf.preepages
  fi
  rm -f /etc/sysctl.conf.~*~ "$SYBASE/$SYBASE_ASE"/*-sysctl.conf.~*~
  FILE=`find "$SYBASE/$SYBASE_ASE"/Scheduler.d -name '*.initial' -printf "%p\n" 2>/dev/null`
  [ -n "$VERBOSE" ] && echo "reset_dbserver_cfg \"$FILE\" \"$FILE\".~*~" 1>&2
  [ -n "$FILE" ] && rm -f ${FILE//.initial/}
}

reset_run_dbserver () {
  return
  #-------------------------------
  local FILE="$SYBASE/$SYBASE_ASE"/install/RUN_DBServer
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "reset_run_dbserver \"$FILE\" \"$FILE\".~*~" 1>&2
  rm -f "$FILE".~*~ "$FILE"_[BM]S.~*~
}

reset_vpd_properties () {
  return
  #-------------------------------
  local FILE="$SYBASE"/vpd.properties
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "reset_vpd_properties \"$FILE\" \"$FILE\".~*~" 1>&2
  rm -f "$FILE".~*~ "$SYBASE"/UAF-2_0/rtlib 2>/dev/null
}

reset_agent_plugin () {
  return
  #-------------------------------
  local FILE="$SYBASE"/UAF-2_0/plugins/com.sybase.ase/agent-plugin.xml
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "reset_agent_plugin \"$FILE\" \"$FILE\".~*~" 1>&2
  rm -f "$FILE".~*~
}

reset_srv_res () {
  return
  #-------------------------------
  local FILE="$SYBASE/$SYBASE_ASE"
  #-------------------------------
  local SRV_KEY="sqlsrv bsrv msrv xp "

  cd "$FILE" 2>/dev/null || return 1
  [ -n "$VERBOSE" ] && echo "reset_srv_res ${SRV_KEY// /.res } ${SRV_KEY// /.res.~*~ }" 1>&2
  rm -f ${SRV_KEY// /.res } ${SRV_KEY// /.res.~*~ }
}

reset_initial_dat () {
  return
  #-------------------------------
  local FILE="$SYBASE/data"
  #-------------------------------
  cd "$FILE" 2>/dev/null || return 1
  [ -n "$VERBOSE" ] && echo "reset_initial_dat \"$FILE\"" 1>&2
  for i in master sybsysdb  sysprocs ; do mv $i.dat $i.dat.initial ; done
}

unlink_jre () {
  return
  #-------------------------------
  local FILE="$SYBASE/shared/sybcentral600/jre"
  #-------------------------------
  cd "${FILE%/*}" 2>/dev/null || return 1
  [ -n "$VERBOSE" ] && echo "unlink_jre \"$FILE\"" 1>&2
  rm -f jre 2>/dev/null
}

#+++++++++++++++++++++++++++++#
#      WEBSRV FUNCTIONS       #
#+++++++++++++++++++++++++++++#

reset_httpd_conf () {
  return
  #-------------------------------
  local FILE="$HTTPD_ROOT/conf/httpd.conf"
  #-------------------------------
  rm -f "$FILE".~*~
  rm -f "$HTTPD_ROOT"/conf/extra/*epages-*.conf.~*~
  FILE=`find "$HTTPD_ROOT"/conf -name '*.initial' -printf "%p\n" 2>/dev/null`
  [ -n "$VERBOSE" ] && echo "reset_httpd_conf \"$FILE\" \"$FILE\".~*~" 1>&2
  [ -n "$FILE" ] && rm -f ${FILE//.initial/}
}

reset_httpd_root () {
  return
  #-------------------------------
  local FILE=`echo $(ls "$HTTPD_ROOT"/bin/*.ept 2>/dev/null)`
  #-------------------------------
  [ -n "$VERBOSE" ] && echo "reset_httpd_root" 1>&2
  [ -n "$FILE" ] && rm -f ${FILE//.ept/} ${FILE//.ept/.~*~}
  rm -f "$HTTPD_ROOT"/bin/httpd-epages

  FILE=`find "$HTTPD_ROOT" -name '*.epx' -printf "%p\n" 2>/dev/null`
  [ -n "$FILE" ] && rm -f ${FILE//.epx/} ${FILE//.epx/.~*~}
}

fi # [ -d "$RPM_INSTALL_PREFIX" ]

###############################################################################
# END                                                                         #
###############################################################################
